import pygame
from pygame.locals import *
import math

pygame.init()

FREQ = 28

noir = 0, 0, 0
blanc = 255, 255, 255
jaune = 255, 255, 0
bleu = 0, 0, 255
transparent = 0, 0, 0, 0
rayon_iris = 10
rayon_deplacement_iris = 10

dali = pygame.image.load("Dali.gif")
daliRect = dali.get_rect()
dimensions = (L,H) = daliRect.size

centre_oeil_gauche = (170,210)
centre_oeil_droit = (280,210)

fenetre = pygame.display.set_mode(dimensions)
clock = pygame.time.Clock()

dali = dali.convert_alpha()

### la surface qui contient l'image d'un iris

iris = pygame.Surface((2 * rayon_iris, 2 * rayon_iris), SRCALPHA)
iris.fill(transparent)
pygame.draw.circle(iris, bleu, (rayon_iris, rayon_iris), rayon_iris)
iris.convert_alpha()

### on commence par quelques fonctions sur les vecteurs 2D! 

def norme(v):
    """ renvoie la norme de v"""
    return math.sqrt(v[0]**2 + v[1]**2)

def prod_ext(k,v) :
    """ multiplie un scalaire et un vecteur"""
    return (k*v[0], k*v[1])

def normalise(v):
    """ renvoie le vecteur unitaire colineaire a v"""
    return prod_ext(1/norme(v), v)

def vecteur(src_pos,dst_pos) :
    """ calcule le vecteur de src a dst"""
    return (dst_pos[0]-src_pos[0], dst_pos[1]-src_pos[1])

def deplace(pos,vect):
    """ renvoie la position obtenue en deplace d'un vecteur donne"""
    return (int(pos[0]+vect[0]), int(pos[1]+vect[1]))

def pos_iris(centre_oeil, pos_souris) :
    """ calcule la position de l'iris en fonction de celle du centre du globe occulaire et de la position de la souris"""
    v = vecteur(centre_oeil, pos_souris)
    if norme(v) > rayon_deplacement_iris :
        v = prod_ext(rayon_deplacement_iris, normalise(v))
    return deplace(centre_oeil, v)


### on passe maintenant a la partie MVC proprement dite
### le monde tout d'abord : ce sera un couple (pos_g,pos_d)
### de positions (la position de chacun des iris)

monde_initial = (centre_oeil_gauche, centre_oeil_droit)

def suivant(monde) : return monde

def fin(monde) : return False

# on passe maintenant a la souris

def souris(fenetre, type, monde) :
    pos_souris = pygame.mouse.get_pos()
    if type == MOUSEMOTION :
        return (pos_iris(centre_oeil_gauche, pos_souris),
                pos_iris(centre_oeil_droit,  pos_souris))
    else:
        return monde

# enfin, la fonction de dessin

def dessine(fenetre, monde) :
    pygame.draw.rect(fenetre, blanc, daliRect)
    for (x,y) in monde :
        fenetre.blit(iris, (x-rayon_iris,y-rayon_iris))
    fenetre.blit(dali, (0,0))


#### la boucle infinie est un copier-coller de l'exercice 3


def boucle_infinie(largeur, hauteur, freq, dessine, suivant, monde_initial, souris, fin):
    clock = pygame.time.Clock()
    monde = monde_initial
    while True:
        for ev in pygame.event.get() :
            if ev.type == QUIT :
                return
            if ev.type in [MOUSEMOTION, MOUSEBUTTONDOWN, MOUSEBUTTONUP] :
                monde = souris(fenetre, ev.type, monde) 
        if fin(monde) :
            return
        dessine(fenetre, monde)
        pygame.display.update()
        monde = suivant(monde)
        clock.tick(freq)


boucle_infinie(L, H, FREQ, dessine, suivant, monde_initial, souris, fin)
